@@ -8,6 +8,7 @@ import socket
 import sys
 import time
 
+import django
 from django.conf import settings
 from django.core.exceptions import ImproperlyConfigured
 from django.core.management.base import BaseCommand, CommandError
@@ -74,8 +75,15 @@ class Command(BaseCommand):
                             help='Specifies an output file to send a copy of all messages (not flushed immediately).')
         parser.add_argument('--print-sql', action='store_true', default=False,
                             help="Print SQL queries as they're executed")
-        parser.add_argument('--cert', dest='cert_path', action="store", type=str,
-                            help='To use SSL, specify certificate path.')
+        cert_group = parser.add_mutually_exclusive_group()
+        cert_group.add_argument('--cert', dest='cert_path', action="store", type=str,
+                                help='Deprecated alias for --cert-file option.')
+        cert_group.add_argument('--cert-file', dest='cert_path', action="store", type=str,
+                                help='SSL .cert file path. If not provided path from --key-file will be selected. '
+                                     'Either --cert-file or --key-file must be provided to use SSL.')
+        parser.add_argument('--key-file', dest='key_file_path', action="store", type=str,
+                            help='SSL .key file path. If not provided path from --cert-file will be selected. '
+                                 'Either --cert-file or --key-file must be provided to use SSL.')
         parser.add_argument('--extra-file', dest='extra_files', action="append", type=str,
                             help='auto-reload whenever the given file changes too (can be specified multiple times)')
         parser.add_argument('--reloader-interval', dest='reloader_interval', action="store", type=int, default=DEFAULT_POLLER_RELOADER_INTERVAL,
@@ -183,9 +191,15 @@ class Command(BaseCommand):
             pm = False
         else:
             # Add pdb middleware if --pdb is specified or if in DEBUG mode
-            middleware = 'django_pdb.middleware.PdbMiddleware'
-            if (pdb_option or ipdb_option or settings.DEBUG) and middleware not in settings.MIDDLEWARE_CLASSES:
-                settings.MIDDLEWARE_CLASSES += (middleware,)
+            if (pdb_option or ipdb_option or settings.DEBUG):
+                middleware = 'django_pdb.middleware.PdbMiddleware'
+                settings_middleware = getattr(settings, 'MIDDLEWARE', None) or settings.MIDDLEWARE_CLASSES
+
+                if middleware not in settings_middleware:
+                    if isinstance(settings_middleware, tuple):
+                        settings_middleware += (middleware,)
+                    else:
+                        settings_middleware += [middleware]
 
             # If --pdb is specified then always break at the start of views.
             # Otherwise break only if a 'pdb' query parameter is set in the url
@@ -239,12 +253,11 @@ class Command(BaseCommand):
                                        % self.addr)
         if not self.addr:
             self.addr = '::1' if self.use_ipv6 else '127.0.0.1'
+            self._raw_ipv6 = True
 
         self.inner_run(options)
 
     def inner_run(self, options):
-        import django
-
         try:
             from werkzeug import run_simple, DebuggedApplication
             from werkzeug.serving import WSGIRequestHandler as _WSGIRequestHandler
@@ -271,7 +284,6 @@ class Command(BaseCommand):
         threaded = options.get('threaded', True)
         use_reloader = options.get('use_reloader', True)
         open_browser = options.get('open_browser', False)
-        cert_path = options.get("cert_path")
         quit_command = (sys.platform == 'win32') and 'CTRL-BREAK' or 'CONTROL-C'
         extra_files = options.get('extra_files', None) or []
         reloader_interval = options.get('reloader_interval', 1)
@@ -295,7 +307,7 @@ class Command(BaseCommand):
             insecure_serving = options.get('insecure_serving', False)
             if use_static_handler and (settings.DEBUG or insecure_serving):
                 handler = StaticFilesHandler(handler)
-        if cert_path:
+        if options.get("cert_path") or options.get("key_file_path"):
             """
             OpenSSL is needed for SSL support.
 
@@ -312,20 +324,15 @@ class Command(BaseCommand):
                                    "required to use runserver_plus with ssl support. "
                                    "Install via pip (pip install pyOpenSSL).")
 
-            dir_path, cert_file = os.path.split(cert_path)
-            if not dir_path:
-                dir_path = os.getcwd()
-            root, ext = os.path.splitext(cert_file)
-            certfile = os.path.join(dir_path, root + ".crt")
-            keyfile = os.path.join(dir_path, root + ".key")
+            certfile, keyfile = self.determine_ssl_files_paths(options)
+            dir_path, root = os.path.split(certfile)
+            root, _ = os.path.splitext(root)
             try:
                 from werkzeug.serving import make_ssl_devcert
-                if os.path.exists(certfile) and \
-                        os.path.exists(keyfile):
-                            ssl_context = (certfile, keyfile)
+                if os.path.exists(certfile) and os.path.exists(keyfile):
+                    ssl_context = (certfile, keyfile)
                 else:  # Create cert, key files ourselves.
-                    ssl_context = make_ssl_devcert(
-                        os.path.join(dir_path, root), host='localhost')
+                    ssl_context = make_ssl_devcert(os.path.join(dir_path, root), host='localhost')
             except ImportError:
                 if self.show_startup_messages:
                     print("Werkzeug version is less than 0.9, trying adhoc certificate.")
@@ -377,16 +384,49 @@ class Command(BaseCommand):
             ssl_context=ssl_context,
         )
 
-    def check_migrations(self):
+    @classmethod
+    def _create_path_with_extension_from(cls, file_path, extension):
+        dir_path, cert_file = os.path.split(file_path)
+        if not dir_path:
+            dir_path = os.getcwd()
+        file_name, _ = os.path.splitext(cert_file)
+        return os.path.join(dir_path, file_name + "." + extension)
+
+    @classmethod
+    def _determine_path_for_file(cls, current_file, other_file, extension):
+        """ Determine path with proper extension. If path is absent then use path from alternative file.
+        If path is relative than use current working directory.
+        :param current_file: path for current file
+        :param other_file: path for alternative file
+        :param extension: expected extension
+        :return: path of this file.
         """
-        Checks to see if the set of migrations on disk matches the
-        migrations in the database. Prints a warning if they don't match.
-        """
-        executor = MigrationExecutor(connections[DEFAULT_DB_ALIAS])
-        plan = executor.migration_plan(executor.loader.graph.leaf_nodes())
-        if plan and self.show_startup_messages:
-            self.stdout.write(self.style.NOTICE("\nYou have unapplied migrations; your app may not work properly until they are applied."))
-            self.stdout.write(self.style.NOTICE("Run 'python manage.py migrate' to apply them.\n"))
+        if current_file is None:
+            return cls._create_path_with_extension_from(other_file, extension)
+        directory, file = os.path.split(current_file)
+        file_name, _ = os.path.splitext(file)
+        if not directory:
+            return cls._create_path_with_extension_from(current_file, extension)
+        else:
+            return os.path.join(directory, file_name + "." + extension)
+
+    @classmethod
+    def determine_ssl_files_paths(cls, options):
+        cert_file = cls._determine_path_for_file(options.get('cert_path'), options.get('key_file_path'), "crt")
+        key_file = cls._determine_path_for_file(options.get('key_file_path'), options.get('cert_path'), "key")
+        return cert_file, key_file
+
+    if django.VERSION[:2] <= (1, 9):
+        def check_migrations(self):
+            """
+            Checks to see if the set of migrations on disk matches the
+            migrations in the database. Prints a warning if they don't match.
+            """
+            executor = MigrationExecutor(connections[DEFAULT_DB_ALIAS])
+            plan = executor.migration_plan(executor.loader.graph.leaf_nodes())
+            if plan and self.show_startup_messages:
+                self.stdout.write(self.style.NOTICE("\nYou have unapplied migrations; your app may not work properly until they are applied."))
+                self.stdout.write(self.style.NOTICE("Run 'python manage.py migrate' to apply them.\n"))
 
 
 def set_werkzeug_log_color():
